/*
 *  GLSpatialObject.h
 *  GLGeometryKit
 *
 *  Created by Jeffrey Early on 9/30/06.
 *  Copyright 2006 Early Innovation. All rights reserved.
 *
 */

/* GLSpatialObject */

// This protocol will be attached to every object (feature) that is considered
// spatial. These objects can be dragged and dropped onto a user-defined
// theme, containing any other spatial objects.
//
// These objects should also register themselves as a particular drag type.

#import <GLGeometryKit/GLDrawContext.h>
#import <GLGeometryKit/GLGeometry.h>

// Specify zero or more items from this list to receive events.
enum {
	GLEventMouseEnteredAndExited = 1 << 1,
	GLEventMouseMoved = 1 << 2,
	GLEventMouseDownAndUp = 1 << 3,
	GLEventMouseDragged = 1 << 4,
};

typedef NSUInteger GLSpatialObjectEventOptions;

@class GLStyle;
@protocol GLSpatialObject

// returns the GLSpatialObjectController that owns the objects, so the plugin can be informed of changes
- (id) objectOwner;

// returns the name of the feature, will be displayed to the user
- (NSString *) featureName;

// returns a more detailed description of the feature, will be displayed to the user
- (NSString *) featureDescription;

// returns the kind of feature, will be displayed to the user
- (NSString *) kind;

// returns a geometry object, used for displaying
- (GLGeometry *) geometry;

// describes how the geometry should be drawn, including the label
- (GLStyle *) style;

// return a dictionary of NSPropertyDescription with key corresponding to the field name.
// These values are also likely to be directly implemented as methods in the
// object, "- (id) value". This will be used for exporting data and building NSPredicates.
- (NSDictionary *) propertiesByName;

// This determines whether or not the MapView handles any events, including selection, for this object.
// If YES is returned, the users mouse over will indicate the object has some clickable function.
// If NO is returned, the MapView will not even *attempt* to change the selection with the object's controller.
- (BOOL) allowsInteraction;

@optional
// I'm not actually sure that this is necessary -- I'm certainly not using it.
// As a way to increase speed, a spatial object can optionally store geometries for a
// number of different resolutions.
- (GLGeometry *) geometryForResolution: (double) resolution;

@end

// *If* you really did define a spatialObject that conforms to the above protocol, then you can call
// this method with a valid drawContext and the object will go ahead and symbolize.
@class GLDrawContext;
@interface NSObject (GLSpatialObjectDrawAdditions)

- (void) didAddToContext: (NSObject <GLDrawContext> *) drawContext;
- (void) didRemoveFromContext: (NSObject <GLDrawContext> *) drawContext;

- (void) drawInContext: (NSObject <GLDrawContext> *) drawContext selected: (BOOL) isSelected;

- (BOOL) isAtPoint: (GLPoint) aPoint inContext: (NSObject <GLDrawContext> *) drawContext selected: (BOOL) isSelected;

- (GLSpatialObjectEventOptions) spatialObjectEventOptions;

- (void) mouseEntered: (NSEvent *) event atPoint: (GLPoint) aPoint inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseExited: (NSEvent *) event atPoint: (GLPoint) aPoint inContext: (NSObject <GLDrawContext> *) drawContext;
- (void) mouseMoved: (NSEvent *) event atPoint: (GLPoint) aPoint inContext: (NSObject <GLDrawContext> *) drawContext;


@end

