//
//  GLMapView.h
//  GLMapKit
//
//  Created by Jeffrey Early on 10/7/06.
//  Copyright 2006 Early Innovation. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <GLGeometryKit/GLGeometryKit.h>

@class GLMapDataSource;

extern NSString *GLMapViewCoordinatesDidChangeNotification;

// Possible actions for the selection box
typedef enum {
	kGLGraphicNoHandle = 0,
    kGLGraphicUpperLeftHandle = 1,
    kGLGraphicUpperMiddleHandle = 2,
    kGLGraphicUpperRightHandle = 3,
    kGLGraphicMiddleLeftHandle = 4,
    kGLGraphicMiddleRightHandle = 5,
    kGLGraphicLowerLeftHandle = 6,
    kGLGraphicLowerMiddleHandle = 7,
    kGLGraphicLowerRightHandle = 8,
	kGLGraphicTranslatingAction = 9,
	kGLGraphicNewBoxAction = 10,
} GLGraphicSelectionBoxActions;

@class GLMapViewAnimator, GLMapController;
@interface GLMapView : NSView <GLDrawContext>
{	
	// holders for the point before and during dragging
	NSPoint downPoint;
	GLPoint currentLatLong;
	// Export tool rectangle
	GLRect exportBox;
	
	BOOL	mouseOverIsActive;
	BOOL	mouseDragIsActive;
	__unsafe_unretained id mouseOverObject;
	
	GLMapDataSource *_mapDataSource;
	
	GLAffineTransform *_translationTransformation;
	GLAffineTransform *_downPointTranslationTransformation;
	GLAffineTransform *_scaleTransformation;
	NSAffineTransform *_contextTransformation;
	
	GLPoint dsCoordinateCache;
	BOOL resetCoordinateCache;
	BOOL inLiveScaling;
	NSMutableDictionary *_mapTiles;
	GLRect _exportBounds;
	GLGraphicSelectionBoxActions currentSelectionBoxAction;
	GLRect downPointExportBox;
	
	GLMapController *mapController;
	
	GLMapViewAnimator *mapAnimator;
	
	GLClippingRegion *clippingRegion;
	GLMapProjection *_mapProjection;
    double scaleDenominator;
	
	BOOL shouldShowCrosshairs;
	
	double tileDrawTime;
	double geomDrawTime;
}

@property double tileDrawTime;
@property double geomDrawTime;

// To be called by the mapController for initialization.
- (void) setInitialLatitudeLongitude: (GLPoint) coords withScale: (double) scale dataSource: (GLMapDataSource *) dataSource;

@property(strong) GLMapViewAnimator *mapAnimator;

/************************************************/
/*		Geographic Coordinates					*/
/************************************************/

#pragma mark -
#pragma mark Geographic Coordinates
#pragma mark

// Sent latitude, longitude in WGS84 with a scale in meters per pixel
- (void) setLongitudeLatitude: (GLPoint) latLong withResolution: (double) scale;

// latitude, longitude in WGS84
@property(nonatomic) GLPoint longitudeLatitude;

// Set scale in meters per pixel -- not to be called by the slider
@property(nonatomic) double scale;

/************************************************/
/*		Projected Coordinates					*/
/************************************************/

#pragma mark -
#pragma mark Projected Coordinates
#pragma mark

// This method does NOT require find integral boundaries for images, nor
// does it call display. Used when initially changing the datasource and the mapAnimator;
- (void) setDSCoordinate: (GLPoint) dsCoordinate dsScale: (double) scale;

// Same as above, but the "center" is determined by the hotspot (in pixel coordinates).
- (void) setDSCoordinate: (GLPoint) dsCoordinate dsScale: (double) scale withHotSpot: (NSPoint) pixelHotSpot;

// Point at the center of the view in projected coordinates
@property(nonatomic, setter=setDSCoordinate:) GLPoint dsCoordinate;

// scale slider sends dsScale in dsUnits/pixel
@property(nonatomic, getter=dsScale,setter=setDSScale:) double dsScale;

// Forces the resolution to be within acceptable range.
- (double) validateResolutionWithMapProjection: (double) resolution;

// During user slider drags and animations, these will be called.
- (void) willStartLiveScaling;
- (void) didEndLiveScaling;

/************************************************/
/*		Geometry Draw Context Protocol			*/
/************************************************/

#pragma mark -
#pragma mark Geometry Draw Context Protocol
#pragma mark

// stores an object representing the projection employed by the draw context, the data source coordinates.
// The geometry will use this object as a weak key to cache a projected version geometry.
@property(nonatomic, strong) GLMapProjection *mapProjection;

// used for scaling the fonts and geometry widths. Scale factor of 1.0 assumes 72 dpi.
@property(readonly) double scaleFactor;

// The currently displayed resolution of the DrawContext in projected units per pixel.
// Used to remove extraneous points in an optimized path, same as projectedResolution
@property(readonly) double projectedUnitsPerPixel;

// The nominal scale of the graphics context, e.g. 1:50,000 would report 50000.
// This *could* simply be the resolution (in meters per pixel) divided by the dpi (converted to meters per pixels).
// This value would change as the user scrolls. So, the compositer might choose something fixed.
@property double scaleDenominator;

// The current display bounds of the DrawContext, in projected coordinates.
@property(readonly) GLRect projectedBounds;

// Transform from projected (data source) coordinates to pixel coordinates
@property(readonly, strong) GLAffineTransform *projectedToPixelTransform;

// The is a more detailed version of the above three properties and includes some precomputed properties useful for 
// speedy clipping.
@property(strong) GLClippingRegion *clippingRegion;

// Should return yes if the compositer will be drawing with the same geometry objects multiple times
@property(readonly) BOOL shouldCacheSpatialObjectGeometry;

// Should return yes if the compositer is actively scaling. This *may* cause quick and dirty representations
// of complex geometry to be rendered in order to improve rendering times.
@property BOOL inLiveScaling;

/************************************************/
/*		Map Tile DataSource						*/
/************************************************/

#pragma mark -
#pragma mark Map Tile DataSource
#pragma mark

// owner of map tiles and definer of coordinates
@property(nonatomic, strong) GLMapDataSource *mapDataSource;

// Stores the current maptile selection
@property(strong) NSMutableDictionary *mapTiles;

// called to determine if the new map tiles need to be request from the datasource
// it requests tiles for the bounds of the display at a certain scale
- (void) checkForMapTiles;

// called by the datasource to pass new map tiles
// might return map tiles of a different scale, if available.
- (void) addMapTiles: (NSArray *) newMapTileArray;


/************************************************/
/*		Pixel Coordinates						*/
/************************************************/

#pragma mark -
#pragma mark Pixel Coordinates
#pragma mark

// units in pixels
@property(nonatomic, strong) GLAffineTransform *translationTransformation;
@property(nonatomic, strong) GLAffineTransform *downPointTranslationTransformation;

// units in pixels/projectedUnit
@property(nonatomic, strong) GLAffineTransform *scaleTransformation;

@property(readonly) GLAffineTransform *pixelToProjectedTransform;

// Forces the transformations to place a representative map tile on pixel boundaries
- (void) integralScaleTransformation: (GLAffineTransform *) aTransform;
- (void) integralTranslationTransformation: (GLAffineTransform *) aTransform;

// If the world wraps, we might need to draw things in some multiple of the primary coordinates,
// and even multiple times. This function returns the values of n you should loop through and multiply by.
// You still need to check that your bounds are on the screen
- (void) translationMinimum: (int *) nmin maximum: (int *) nmax distance: (int *) pixelWidth;

// Because we try to draw everything on pixel bounds, during scaling the center of the screen
// can actually shift quite a bit. Thus, we log the 'true' spot before scaling and use that one.
@property GLPoint dsCoordinateCache;
@property BOOL resetCoordinateCache;

- (GLPoint) viewPointFromLatLon:  (GLPoint) latLonPoint;
- (GLPoint) latLonFromViewPoint: (GLPoint) pixelPoint;

/************************************************/
/*		Mouse Events							*/
/************************************************/

#pragma mark -
#pragma mark Mouse Events
#pragma mark

@property BOOL mouseOverIsActive;
@property BOOL mouseDragIsActive;
@property(unsafe_unretained) id mouseOverObject;

// Given a point in pixel coordinates, is that point over any spatial object *that allows interaction*
- (BOOL) isPixelPointOverSpatialObject: (GLPoint) mouseOverPoint forEvent: (NSEvent *) event;

// Array of objects *that allow interaction* directly underneath the point
- (NSArray *) spatialObjectsAtPixelPoint: (GLPoint) mouseOverPoint;

/************************************************/
/*		Export									*/
/************************************************/

#pragma mark -
#pragma mark Export
#pragma mark

- (void) selectedToolDidChange;
- (void) export;
@property GLRect exportBounds;
GLRect makePositiveRect(GLRect aRect);

- (void)drawHandlesForRect:(NSRect) bounds;
- (void)drawHandleAtPoint:(NSPoint)point;
- (int)handleUnderPoint:(NSPoint)point forRect:(NSRect) bounds;
- (BOOL)isHandleAtPoint:(NSPoint)handlePoint underPoint:(NSPoint)point;
@property GLGraphicSelectionBoxActions currentSelectionBoxAction;
@property GLRect downPointExportBox;

- (void) transformExportBoxFromProjection: (GLMapProjection *) oldProjection toProjection: (GLMapProjection *) newProjection;

@property(nonatomic) BOOL shouldShowCrosshairs;


/************************************************/
/*		Spatial Object Observation				*/
/************************************************/

#pragma mark -
#pragma mark Spatial Object Observation
#pragma mark

@property(strong) GLMapController *mapController;

@property(readonly) NSArray *spatialObjects;
@property(readonly) NSIndexSet *spatialObjectSelectionIndexes;

// Called by the mapController after spatial objects have been added and/or removed, and when selection indexes change.
- (void) spatialObjectChangesDidFinish;
- (void)stopObservingSpatialObjects:(NSArray *)spatialObjects;
- (void)startObservingSpatialObjects:(NSArray *)spatialObjects;

@end
